import re

from smartmin.views import (
    SmartCreateView,
    SmartCRUDL,
    SmartFormView,
    SmartListView,
    SmartReadView,
    SmartTemplateView,
    SmartUpdateView,
)

from django import forms
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from .forms import CreateOrgLoginForm, OrgForm
from .models import Invitation, Org, OrgBackend, OrgBackground, TaskState


class OrgPermsMixin(object):
    """
    Get the organisation and the user within the inheriting view so that it be
    come easy to decide whether this user has a certain permission for that
    particular organization to perform the view's actions
    """

    def get_user(self):
        return self.request.user

    def derive_org(self):
        return self.request.org

    def pre_process(self, request, *args, **kwargs):
        user = self.get_user()
        org = self.derive_org()

        if user.is_superuser:
            return None

        if not org:
            return HttpResponseRedirect(reverse("orgs.org_choose"))

        return None

    def has_org_perm(self, permission):
        (app_label, codename) = permission.split(".")

        if self.get_user().is_superuser:
            return True

        if self.get_user().is_anonymous:
            return False

        if self.org:
            org_group = self.get_user().get_org_group()
            if org_group:
                if org_group.permissions.filter(content_type__app_label=app_label, codename=codename).exists():
                    return True

        return False

    def has_permission(self, request, *args, **kwargs):
        """
        Figures out if the current user has permissions for this view.
        """
        self.kwargs = kwargs
        self.args = args
        self.request = request
        self.org = self.derive_org()

        if self.get_user().is_superuser:
            return True

        if self.get_user().has_perm(self.permission):
            return True

        return self.has_org_perm(self.permission)


class OrgObjPermsMixin(OrgPermsMixin):
    def get_object_org(self):
        return self.get_object().org

    def has_org_perm(self, codename):
        has_org_perm = super(OrgObjPermsMixin, self).has_org_perm(codename)

        if has_org_perm:
            user = self.get_user()

            if user.is_anonymous:
                return True
            return user.get_org() == self.get_object_org()

        return False

    def has_permission(self, request, *args, **kwargs):
        has_perm = super(OrgObjPermsMixin, self).has_permission(request, *args, **kwargs)

        if has_perm:
            user = self.get_user()

            # user has global permission
            if user.has_perm(self.permission):
                return True

            return user.get_org() == self.get_object_org()

        return False


class InferOrgMixin(object):
    @classmethod
    def derive_url_pattern(cls, path, action):
        return r"^%s/%s/$" % (path, action)

    def get_object(self, *args, **kwargs):
        return self.request.org


class OrgCRUDL(SmartCRUDL):
    actions = (
        "create",
        "list",
        "update",
        "choose",
        "home",
        "edit",
        "manage_accounts",
        "create_login",
        "join",
        "chooser",
    )
    model = Org

    class Chooser(SmartTemplateView):
        permission = False
        template_name = getattr(settings, "SITE_CHOOSER_TEMPLATE", "orgs/org_chooser.html")

        def get_context_data(self, **kwargs):
            all_orgs = Org.objects.filter(is_active=True).order_by("name")

            # populate a 'host' attribute on each org so we can link off to them
            for org in all_orgs:
                org.host = org.build_host_link()

            return dict(orgs=all_orgs)

    class Create(SmartCreateView):
        form_class = OrgForm
        fields = ("name", "language", "timezone", "subdomain", "domain", "logo", "administrators")

    class Update(SmartUpdateView):
        form_class = OrgForm
        fields = ("is_active", "name", "language", "timezone", "subdomain", "domain", "logo", "administrators")

    class List(SmartListView):
        fields = ("name", "timezone", "created_on", "modified_on")
        ordering = ("name",)

    class Choose(SmartFormView):
        class ChooseForm(forms.Form):
            def __init__(self, *args, **kwargs):
                self.user = kwargs["user"]
                del kwargs["user"]

                super(OrgCRUDL.Choose.ChooseForm, self).__init__(*args, **kwargs)
                self.fields["organization"].queryset = self.user.get_user_orgs()

            organization = forms.ModelChoiceField(queryset=Org.objects.filter(id__lte=-1), empty_label=None)

        form_class = ChooseForm
        fields = ("organization",)
        title = _("Select your Organization")

        def pre_process(self, request, *args, **kwargs):
            if self.request.user.is_authenticated:
                user_orgs = self.request.user.get_user_orgs()

                if self.request.user.is_superuser:
                    return HttpResponseRedirect(reverse("orgs.org_list"))

                elif not user_orgs:
                    messages.info(
                        request,
                        _("Your account is not associated to an " "organization. Please Contact the adminstrator."),
                    )
                    return HttpResponseRedirect(reverse("users.user_login"))

                elif user_orgs.count() == 1:
                    org = user_orgs[0]
                    if org and self.request.org:
                        if self.request.org == org:
                            self.request.session["org_id"] = org.pk
                            return HttpResponseRedirect(self.get_success_url())

            return None

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.Choose, self).get_context_data(**kwargs)

            context["orgs"] = self.request.user.get_user_orgs()
            return context

        def has_permission(self, request, *args, **kwargs):
            return self.request.user.is_authenticated

        def get_form_kwargs(self):
            kwargs = super(OrgCRUDL.Choose, self).get_form_kwargs()
            kwargs["user"] = self.request.user
            return kwargs

        def form_valid(self, form):
            org = form.cleaned_data["organization"]

            if org in self.request.user.get_user_orgs():
                self.request.session["org_id"] = org.pk
                self.request.org = org

            return HttpResponseRedirect(org.build_host_link() + self.get_success_url())

        def get_success_url(self):
            return getattr(settings, "SITE_USER_HOME", reverse("orgs.org_home"))

    class Home(InferOrgMixin, OrgPermsMixin, SmartReadView):
        title = _("Your Organization")
        fields = ("name", "subdomain")

    class Edit(InferOrgMixin, OrgPermsMixin, SmartUpdateView):
        title = _("Your Organization")
        success_url = "@orgs.org_home"
        fields = ("name",)

        def derive_fields(self):
            fields = super(OrgCRUDL.Edit, self).derive_fields()
            is_super = self.request.user.is_superuser

            config_fields = getattr(settings, "ORG_CONFIG_FIELDS", [])
            for config_field in config_fields:
                read_only = config_field.get("read_only", False)
                if is_super or read_only or not config_field.get("superuser_only", False):
                    fields.append("common.%s" % config_field["name"])

            backends = self.get_object().backends.filter(is_active=True)
            backends = backends.exclude(api_token="").exclude(api_token=None).values_list("slug", flat=True)
            for backend_slug in backends:
                config_fields = getattr(settings, "BACKENDS_ORG_CONFIG_FIELDS", [])
                for config_field in config_fields:
                    read_only = config_field.get("read_only", False)
                    if is_super or read_only or not config_field.get("superuser_only", False):
                        fields.append("%s.%s" % (backend_slug, config_field["name"]))
            return fields

        def get_form(self):
            form = super(OrgCRUDL.Edit, self).get_form()
            is_super = self.request.user.is_superuser

            # add all our configured org fields as well
            config_fields = getattr(settings, "ORG_CONFIG_FIELDS", [])
            for config_field in config_fields:
                read_only = config_field.get("read_only", False)
                if is_super or read_only or not config_field.get("superuser_only", False):
                    name = config_field["name"]
                    field_name = "common.%s" % name
                    if name.startswith("has_") or name.startswith("is_"):
                        form.fields[field_name] = forms.BooleanField(**config_field["field"])
                    else:
                        form.fields[field_name] = forms.CharField(**config_field["field"])

                    if not is_super and read_only:
                        form.fields[field_name].widget.attrs["readonly"] = "readonly"
                        form.fields[field_name].required = False

            backends = self.get_object().backends.filter(is_active=True)
            backends = backends.exclude(api_token="").exclude(api_token=None).values_list("slug", flat=True)
            for backend_slug in backends:
                config_fields = getattr(settings, "BACKENDS_ORG_CONFIG_FIELDS", [])
                for config_field in config_fields:
                    read_only = config_field.get("read_only", False)
                    if is_super or read_only or not config_field.get("superuser_only", False):
                        name = config_field["name"]
                        field_name = "%s.%s" % (backend_slug, name)
                        if name.startswith("has_") or name.startswith("is_"):
                            form.fields[field_name] = forms.BooleanField(**config_field["field"])
                        else:
                            form.fields[field_name] = forms.CharField(**config_field["field"])

                        if not is_super and read_only:
                            form.fields[field_name].widget.attrs["readonly"] = "readonly"
                            form.fields[field_name].required = False
            return form

        def pre_save(self, obj):
            obj = super(OrgCRUDL.Edit, self).pre_save(obj)
            cleaned = self.form.cleaned_data
            is_super = self.request.user.is_superuser

            config_fields = getattr(settings, "ORG_CONFIG_FIELDS", [])
            for config_field in config_fields:
                read_only = config_field.get("read_only", False)
                if is_super or (not config_field.get("superuser_only", False) and not read_only):
                    name = "common.%s" % config_field["name"]
                    obj.set_config(name, cleaned.get(name, None))

            backends = self.get_object().backends.filter(is_active=True)
            backends = backends.exclude(api_token="").exclude(api_token=None).values_list("slug", flat=True)
            for backend_slug in backends:
                config_fields = getattr(settings, "BACKENDS_ORG_CONFIG_FIELDS", [])
                for config_field in config_fields:
                    read_only = config_field.get("read_only", False)
                    if is_super or (not config_field.get("superuser_only", False) and not read_only):
                        name = "%s.%s" % (backend_slug, config_field["name"])
                        obj.set_config(name, cleaned.get(name, None))
            return obj

        def derive_initial(self):
            initial = super(OrgCRUDL.Edit, self).derive_initial()
            is_super = self.request.user.is_superuser

            config_fields = getattr(settings, "ORG_CONFIG_FIELDS", [])
            for config_field in config_fields:
                read_only = config_field.get("read_only", False)
                if is_super or read_only or not config_field.get("superuser_only", False):
                    name = "common.%s" % config_field["name"]
                    initial[name] = self.object.get_config(name)

            backends = self.get_object().backends.filter(is_active=True)
            backends = backends.exclude(api_token="").exclude(api_token=None).values_list("slug", flat=True)
            for backend_slug in backends:
                config_fields = getattr(settings, "BACKENDS_ORG_CONFIG_FIELDS", [])
                for config_field in config_fields:
                    read_only = config_field.get("read_only", False)
                    if is_super or read_only or not config_field.get("superuser_only", False):
                        name = "%s.%s" % (backend_slug, config_field["name"])
                        initial[name] = self.object.get_config(name)

            return initial

        def get_object(self, *args, **kwargs):
            return self.request.org

    class ManageAccounts(InferOrgMixin, OrgPermsMixin, SmartUpdateView):
        class InviteForm(forms.ModelForm):
            emails = forms.CharField(label=_("Invite people to your organization"), required=False)
            user_group = forms.ChoiceField(
                choices=(("A", _("Administrators")), ("E", _("Editors"))),
                required=True,
                initial="E",
                label=_("User group"),
            )

            def clean_emails(self):
                emails = self.cleaned_data["emails"].lower().strip()
                if emails:
                    email_list = emails.split(",")
                    for email in email_list:
                        try:
                            validate_email(email)
                        except ValidationError:
                            raise forms.ValidationError(_("One of the emails you entered is invalid."))
                return emails

            class Meta:
                model = Invitation
                fields = ("emails", "user_group")

        form_class = InviteForm
        success_url = "@orgs.org_home"
        success_message = ""
        GROUP_LEVELS = ("administrators", "editors")

        def derive_title(self):
            return _("Manage %(name)s Accounts") % {"name": self.get_object().name}

        def add_check_fields(self, form, objects, org_id, field_dict):
            for obj in objects:
                fields = []
                for grp_level in self.GROUP_LEVELS:
                    check_field = forms.BooleanField(required=False)
                    field_name = "%s_%d" % (grp_level, obj.id)

                    form.fields[field_name] = check_field
                    fields.append(field_name)

                field_dict[obj] = fields

        def derive_initial(self):
            self.org_users = self.get_object().get_org_users()

            initial = dict()
            for grp_level in self.GROUP_LEVELS:
                if grp_level == "administrators":
                    assigned_users = self.get_object().get_org_admins()
                if grp_level == "editors":
                    assigned_users = self.get_object().get_org_editors()

                for obj in assigned_users:
                    key = "%s_%d" % (grp_level, obj.id)
                    initial[key] = True

            return initial

        def get_form(self):
            form = super(OrgCRUDL.ManageAccounts, self).get_form()
            self.group_fields = dict()
            self.add_check_fields(form, self.org_users, self.get_object().pk, self.group_fields)

            return form

        def post_save(self, obj):
            obj = super(OrgCRUDL.ManageAccounts, self).post_save(obj)

            cleaned_data = self.form.cleaned_data
            user = self.request.user
            org = self.get_object()

            user_group = cleaned_data["user_group"]

            emails = cleaned_data["emails"].lower().strip()
            email_list = emails.split(",")

            if emails:
                for email in email_list:

                    # if they already have an invite, update it
                    invites = Invitation.objects.filter(email=email, org=org).order_by("-pk")
                    invitation = invites.first()

                    if invitation:

                        # remove any old invites
                        invites.exclude(pk=invitation.pk).delete()

                        invitation.user_group = user_group
                        invitation.is_active = True
                        invitation.save()
                    else:
                        invitation = Invitation.objects.create(
                            email=email, org=org, user_group=user_group, created_by=user, modified_by=user
                        )

                    invitation.send_invitation()

            # remove all the org users
            for user in self.get_object().get_org_admins():
                if user != self.request.user:
                    self.get_object().administrators.remove(user)
                else:
                    self.get_object().administrators.add(user)
            for user in self.get_object().get_org_editors():
                self.get_object().editors.remove(user)

            # now update the org accounts
            for field in self.form.fields:
                if self.form.cleaned_data[field]:
                    matcher = re.match(r"(\w+)_(\d+)", field)
                    if matcher:
                        user_type = matcher.group(1)
                        user_id = matcher.group(2)
                        user = User.objects.get(pk=user_id)
                        if user_type == "administrators":
                            self.get_object().administrators.add(user)
                        if user_type == "editors":
                            self.get_object().editors.add(user)

            # update our org users after we've removed them
            self.org_users = self.get_object().get_org_users()

            return obj

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.ManageAccounts, self).get_context_data(**kwargs)
            org = self.get_object()
            context["org"] = org
            context["org_users"] = self.org_users
            context["group_fields"] = self.group_fields
            context["invites"] = Invitation.objects.filter(org=org, is_active=True)

            return context

    class CreateLogin(SmartFormView):
        title = ""
        form_class = CreateOrgLoginForm
        success_message = ""
        submit_button_name = _("Create")
        permission = False

        def pre_process(self, request, *args, **kwargs):
            secret = self.kwargs.get("secret")
            org = self.get_object()
            if not org:
                messages.info(
                    request, _("Your invitation link is invalid. Please " "contact your organization administrator.")
                )
                return HttpResponseRedirect("/")

            elif request.org != org:
                redirect_path = reverse("orgs.org_create_login", args=[secret])
                redirect_path = org.build_host_link() + redirect_path
                return HttpResponseRedirect(redirect_path)

            return None

        def form_valid(self, form):
            user = Org.create_user(self.form.cleaned_data["email"], self.form.cleaned_data["password"])
            user.first_name = self.form.cleaned_data["first_name"]
            user.last_name = self.form.cleaned_data["last_name"]
            user.save()

            invitation = self.get_invitation()

            # log the user in
            user = authenticate(username=user.username, password=self.form.cleaned_data["password"])
            login(self.request, user)

            obj = self.get_object()
            if invitation.user_group == "A":
                obj.administrators.add(user)
            elif invitation.user_group == "E":
                obj.editors.add(user)

            # make the invitation inactive
            invitation.is_active = False
            invitation.save()

            return super(OrgCRUDL.CreateLogin, self).form_valid(form)

        @classmethod
        def derive_url_pattern(cls, path, action):
            return r"^%s/%s/(?P<secret>\w+)/$" % (path, action)

        def get_invitation(self, **kwargs):
            invitation = None
            secret = self.kwargs.get("secret")
            invitations = Invitation.objects.filter(secret=secret, is_active=True)
            if invitations:
                invitation = invitations[0]
            return invitation

        def get_object(self, **kwargs):
            invitation = self.get_invitation()
            if invitation:
                return invitation.org

        def derive_title(self):
            org = self.get_object()
            return _("Join %(name)s") % {"name": org.name}

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.CreateLogin, self).get_context_data(**kwargs)

            context["secret"] = self.kwargs.get("secret")
            context["org"] = self.get_object()

            return context

        def get_success_url(self):
            return getattr(settings, "SITE_USER_HOME", reverse("orgs.org_home"))

    class Join(SmartUpdateView):
        class JoinForm(forms.ModelForm):
            class Meta:
                model = Org
                fields = ()

        success_message = ""
        form_class = JoinForm
        submit_button_name = _("Join")
        permission = False

        def pre_process(self, request, *args, **kwargs):
            secret = self.kwargs.get("secret")

            org = self.get_object()
            if not org:
                messages.info(
                    request, _("Your invitation link has expired. Please " "contact your organization administrator.")
                )
                return HttpResponseRedirect("/")
            elif request.org != org:

                redirect_path = org.build_host_link() + reverse("orgs.org_join", args=[secret])
                return HttpResponseRedirect(redirect_path)

            if not request.user.is_authenticated:
                return HttpResponseRedirect(reverse("orgs.org_create_login", args=[secret]))
            return None

        def derive_title(self):
            org = self.get_object()
            return _("Join %(name)s") % {"name": org.name}

        def save(self, org):
            org = self.get_object()
            invitation = self.get_invitation()
            if org:
                if invitation.user_group == "A":
                    org.administrators.add(self.request.user)
                elif invitation.user_group == "E":
                    org.editors.add(self.request.user)

                # make the invitation inactive
                invitation.is_active = False
                invitation.save()

                # set the active org on this user
                self.request.user.set_org(org)
                self.request.session["org_id"] = org.pk

        @classmethod
        def derive_url_pattern(cls, path, action):
            return r"^%s/%s/(?P<secret>\w+)/$" % (path, action)

        def get_invitation(self, **kwargs):
            invitation = None
            secret = self.kwargs.get("secret")
            invitations = Invitation.objects.filter(secret=secret, is_active=True)
            if invitations:
                invitation = invitations[0]
            return invitation

        def get_object(self, **kwargs):
            invitation = self.get_invitation()
            if invitation:
                return invitation.org

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.Join, self).get_context_data(**kwargs)

            context["org"] = self.get_object()
            return context

        def get_success_url(self):
            return getattr(settings, "SITE_USER_HOME", reverse("orgs.org_home"))


class OrgBackgroundCRUDL(SmartCRUDL):
    model = OrgBackground
    actions = ("create", "update", "list")

    class Update(OrgObjPermsMixin, SmartUpdateView):
        fields = ("is_active", "name", "background_type", "image")

    class List(OrgPermsMixin, SmartListView):
        fields = ("name", "background_type")
        ordering = ("name",)

        def derive_fields(self):
            if self.request.user.is_superuser:
                return ("org", "name", "background_type")
            return ("name", "background_type")

        def get_background_type(self, obj):
            return obj.get_background_type_display()

        def get_queryset(self, **kwargs):
            queryset = super(OrgBackgroundCRUDL.List, self).get_queryset(**kwargs)

            if not self.get_user().is_superuser:
                queryset = queryset.filter(org=self.derive_org())

            return queryset

    class Create(OrgPermsMixin, SmartCreateView):
        def derive_fields(self):
            if self.request.user.is_superuser:
                return ("org", "name", "background_type", "image")
            return ("name", "background_type", "image")

        def pre_save(self, obj):
            obj = super(OrgBackgroundCRUDL.Create, self).pre_save(obj)

            if not self.get_user().is_superuser:
                org = self.derive_org()
                if org:
                    obj.org = org

            return obj


class TaskCRUDL(SmartCRUDL):
    actions = ("list",)
    model = TaskState
    model_name = _("Task")
    path = "task"

    class List(SmartListView):
        title = _("Tasks")
        link_fields = ("org",)
        default_order = ("org__name", "task_key")
        ordering = ("org__name", "task_key")

        def lookup_field_link(self, context, field, obj):
            if field == "org":
                return reverse("orgs.org_update", args=[obj.org_id])
            else:
                return super(TaskCRUDL.List, self).lookup_field_link(context, field, obj)

        def derive_queryset(self, **kwargs):
            qs = super().derive_queryset(**kwargs).filter(org__is_active=True)

            if self.request.GET.get("failing"):
                qs = qs.filter(is_failing=True)

            return qs


class OrgBackendForm(forms.ModelForm):
    backend_type = forms.ChoiceField(choices=settings.DATA_API_BACKEND_TYPES)

    class Meta:
        model = OrgBackend
        fields = ("is_active", "org", "slug", "backend_type", "host", "api_token")


class OrgBackendCRUDL(SmartCRUDL):
    model = OrgBackend
    actions = ("create", "update", "list")

    class Update(OrgObjPermsMixin, SmartUpdateView):
        form_class = OrgBackendForm
        fields = ("is_active", "slug", "backend_type", "host", "api_token")

    class Create(OrgPermsMixin, SmartCreateView):
        form_class = OrgBackendForm
        fields = ("org", "slug", "backend_type", "host", "api_token")

    class List(OrgPermsMixin, SmartListView):
        fields = ("org", "slug", "backend_type", "modified_on", "created_on")
        ordering = ("org__name", "slug")

        def get_queryset(self, **kwargs):
            queryset = super(OrgBackendCRUDL.List, self).get_queryset(**kwargs)

            if self.derive_org():
                queryset = queryset.filter(org=self.derive_org())

            return queryset



from smartmin.models import SmartModel

from django.db import models
from django.utils.translation import gettext_lazy as _

from dash.orgs.models import Org


class Tag(SmartModel):

    name = models.CharField(max_length=64, help_text=_("The name of this tag"))
    org = models.ForeignKey(Org, on_delete=models.PROTECT)

    def __str__(self) -> str:
        return self.name

    class Meta:
        unique_together = ("name", "org")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-21 09:34


import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("orgs", "0022_populate_rapidpro_specific_config"),
    ]

    operations = [
        migrations.CreateModel(
            name="OrgBackend",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                        editable=False,
                        help_text="When this item was originally created",
                    ),
                ),
                (
                    "modified_on",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                        editable=False,
                        help_text="When this item was last modified",
                    ),
                ),
                (
                    "api_token",
                    models.CharField(
                        blank=True, help_text="The API token for this backend", max_length=128, null=True
                    ),
                ),
                ("host", models.CharField(blank=True, max_length=128, null=True)),
                ("slug", models.CharField(max_length=16)),
                ("backend_type", models.CharField(blank=True, max_length=256, null=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        related_name="orgs_orgbackend_creations",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        related_name="orgs_orgbackend_modifications",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("org", models.ForeignKey(on_delete=models.PROTECT, related_name="backends", to="orgs.Org")),
            ],
            options={"abstract": False},
        )
    ]



from datetime import datetime

import phonenumbers

from django import template
from django.utils import timezone

register = template.Library()


@register.simple_tag()
def display_time(text_timestamp, org, time_format=None):

    if not time_format:
        time_format = "%b %d, %Y %H:%M"

    parsed_time = datetime.strptime(text_timestamp, "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
    output_time = parsed_time.astimezone(org.timezone)

    return output_time.strftime(time_format)


@register.simple_tag()
def national_phone(number_str):
    if number_str and number_str[0] == "+":
        try:
            return phonenumbers.format_number(
                phonenumbers.parse(number_str, None), phonenumbers.PhoneNumberFormat.NATIONAL
            )
        except Exception:
            # number didn't parse, return it raw
            return number_str

    return number_str



# -*- coding: utf-8 -*-


from django.db import migrations


class Migration(migrations.Migration):
    def move_images(apps, schema_editor):
        # for each story we want to move any image currently on it and instead create a StoryImage for it
        Story = apps.get_model("stories", "Story")
        StoryImage = apps.get_model("stories", "StoryImage")

        for story in Story.objects.exclude(image=None):
            StoryImage.objects.create(
                story=story, image=story.image, created_by=story.created_by, modified_by=story.modified_by
            )

    dependencies = [("stories", "0007_storyimage")]

    operations = [migrations.RunPython(move_images)]



from compressor.management.commands.compress import Command as CompressCommand


class Command(CompressCommand):
    """
    Defer to the real compress command in django_compressor. This subclass is only for backward compatibility so that
    we can still invoke it as hamlcompress
    """

    pass



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("dashblocks", "0005_auto_20140904_0957")]

    operations = [
        migrations.AlterField(
            model_name="dashblock",
            name="created_by",
            field=models.ForeignKey(
                related_name="dashblocks_dashblock_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="dashblock",
            name="modified_by",
            field=models.ForeignKey(
                related_name="dashblocks_dashblock_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="dashblockimage",
            name="created_by",
            field=models.ForeignKey(
                related_name="dashblocks_dashblockimage_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="dashblockimage",
            name="dashblock",
            field=models.ForeignKey(related_name="images", on_delete=models.PROTECT, to="dashblocks.DashBlock"),
        ),
        migrations.AlterField(
            model_name="dashblockimage",
            name="modified_by",
            field=models.ForeignKey(
                related_name="dashblocks_dashblockimage_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="dashblocktype",
            name="created_by",
            field=models.ForeignKey(
                related_name="dashblocks_dashblocktype_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="dashblocktype",
            name="modified_by",
            field=models.ForeignKey(
                related_name="dashblocks_dashblocktype_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
    ]



from functools import partial

from smartmin.models import SmartModel

from django.db import models
from django.utils.translation import gettext_lazy as _

from dash.orgs.models import Org
from dash.utils import generate_file_path


class DashBlockType(SmartModel):
    """
    Dash Block Types just group fields by a slug.. letting you do lookups by
    type. In the future it may be nice to specify which fields should be
    displayed when creating fields of a new type.
    """

    name = models.CharField(max_length=75, unique=True, help_text=_("The human readable name for this content type"))
    slug = models.SlugField(
        max_length=50,
        unique=True,
        help_text=_("The slug to idenfity this content type, used with the " "template tags"),
    )
    description = models.TextField(
        blank=True,
        null=True,
        help_text=_("A description of where this content type is used on the " "site and how it will be dsiplayed"),
    )

    has_title = models.BooleanField(default=True, help_text=_("Whether this content should include a title"))
    has_image = models.BooleanField(default=True, help_text=_("Whether this content should include an image"))
    has_rich_text = models.BooleanField(
        default=True, help_text=_("Whether this content should use a rich HTML editor")
    )
    has_summary = models.BooleanField(default=True, help_text=_("Whether this content should include a summary field"))
    has_link = models.BooleanField(default=True, help_text=_("Whether this content should include a link"))
    has_gallery = models.BooleanField(
        default=False, help_text=_("Whether this content should allow upload of additional " "images, ie a gallery")
    )
    has_color = models.BooleanField(default=False, help_text=_("Whether this content has a color field"))
    has_video = models.BooleanField(
        default=False, help_text=_("Whether this content should allow setting a YouTube id")
    )
    has_tags = models.BooleanField(default=False, help_text=_("Whether this content should allow tags"))

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class DashBlock(SmartModel):
    """
    A DashBlock is just a block of content, organized by type and priority.
    All fields are optional letting you use them for different things.
    """

    dashblock_type = models.ForeignKey(
        DashBlockType,
        on_delete=models.PROTECT,
        verbose_name=_("Content Type"),
        help_text=_("The category, or type for this content block"),
    )

    title = models.CharField(
        max_length=255, blank=True, null=True, help_text=_("The title for this block of content, optional")
    )
    summary = models.TextField(blank=True, null=True, help_text=_("The summary for this item, should be short"))
    content = models.TextField(blank=True, null=True, help_text=_("The body of text for this content block, optional"))
    image = models.ImageField(
        blank=True,
        null=True,
        upload_to=partial(generate_file_path, "dashblocks"),
        help_text=_("Any image that should be displayed with this content " "block, optional"),
    )
    color = models.CharField(
        blank=True,
        null=True,
        max_length=16,
        help_text=_("A background color to use for the image, in the " "format: #rrggbb"),
    )
    link = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        help_text=_("Any link that should be associated with this content " "block, optional"),
    )
    video_id = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        help_text=_("The id of the YouTube video that should be linked to " "this item"),
    )
    tags = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        help_text=_(
            "Any tags for this content block, separated by spaces, "
            "can be used to do more advanced filtering, optional"
        ),
    )
    priority = models.IntegerField(
        default=0, help_text=_("The priority for this block, higher priority blocks " "come first")
    )

    org = models.ForeignKey(
        Org, on_delete=models.PROTECT, help_text=_("The organization this content block belongs to")
    )

    def teaser(self, field, length):
        words = field.split(" ")

        if len(words) < length:
            return field
        else:
            return " ".join(words[:length]) + " ..."

    def long_content_teaser(self):
        return self.teaser(self.content, 100)

    def short_content_teaser(self):
        return self.teaser(self.content, 40)

    def long_summary_teaser(self):
        return self.teaser(self.summary, 100)

    def short_summary_teaser(self):
        return self.teaser(self.summary, 40)

    def space_tags(self):
        """
        If we have tags set, then adds spaces before and after to allow for SQL
        querying for them.
        """
        if self.tags and self.tags.strip():
            self.tags = " " + self.tags.strip().lower() + " "

    def sorted_images(self):
        return self.images.filter(is_active=True).order_by("-priority")

    def __str__(self):
        if self.dashblock_type.has_title:
            return self.title
        return "%s - %d" % (self.dashblock_type, self.pk)

    class Meta:
        ordering = ["dashblock_type", "title"]


class DashBlockImage(SmartModel):
    dashblock = models.ForeignKey(DashBlock, on_delete=models.PROTECT, related_name="images")
    image = models.ImageField(
        upload_to=partial(generate_file_path, "dashblock_images/"), width_field="width", height_field="height"
    )
    caption = models.CharField(max_length=64)
    priority = models.IntegerField(default=0, blank=True, null=True)
    width = models.IntegerField()
    height = models.IntegerField()

    def __str__(self):
        return self.image.url



from smartmin.views import SmartCreateView, SmartCRUDL, SmartListView, SmartUpdateView

from django import forms
from django.core.validators import validate_image_file_extension
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from dash.categories.fields import CategoryChoiceField
from dash.orgs.views import OrgObjPermsMixin, OrgPermsMixin

from .models import Category, Story, StoryImage


class StoryForm(forms.ModelForm):
    category = CategoryChoiceField(Category.objects.none())

    def __init__(self, *args, **kwargs):
        self.org = kwargs["org"]
        del kwargs["org"]
        super(StoryForm, self).__init__(*args, **kwargs)

        # We show all categories even inactive one in the dropdown
        qs = Category.objects.filter(org=self.org).order_by("name")
        self.fields["category"].queryset = qs

    class Meta:
        model = Story
        fields = (
            "is_active",
            "title",
            "featured",
            "summary",
            "content",
            "attachment",
            "written_by",
            "audio_link",
            "video_id",
            "tags",
            "category",
        )


class StoryCRUDL(SmartCRUDL):
    model = Story
    actions = ("create", "update", "list", "images")

    class Update(OrgObjPermsMixin, SmartUpdateView):
        form_class = StoryForm
        fields = (
            "is_active",
            "title",
            "featured",
            "summary",
            "content",
            "attachment",
            "written_by",
            "audio_link",
            "video_id",
            "tags",
            "category",
        )

        def pre_save(self, obj):
            obj = super(StoryCRUDL.Update, self).pre_save(obj)
            obj.audio_link = Story.format_audio_link(obj.audio_link)
            obj.tags = Story.space_tags(obj.tags)
            return obj

        def get_form_kwargs(self):
            kwargs = super(StoryCRUDL.Update, self).get_form_kwargs()
            kwargs["org"] = self.request.org
            return kwargs

    class List(OrgPermsMixin, SmartListView):
        fields = ("title", "images", "featured", "has_report", "created_on")
        search_fields = ("title__icontains",)
        link_fields = ("title", "images")
        default_order = ("-created_on",)
        ordering = ("-created_on",)

        def get_has_report(self, obj):
            if obj.attachment:
                return _("Yes")
            return _("No")

        def get_featured(self, obj):
            if obj.featured:
                return _("Yes")
            return _("No")

        def lookup_field_link(self, context, field, obj):
            if field == "images":
                return reverse("stories.story_images", args=[obj.pk])
            else:
                return super(StoryCRUDL.List, self).lookup_field_link(context, field, obj)

        def get_images(self, obj):
            return obj.images.count()

        def get_queryset(self, **kwargs):
            queryset = super(StoryCRUDL.List, self).get_queryset(**kwargs)
            queryset = queryset.filter(org=self.derive_org())

            return queryset

    class Images(OrgObjPermsMixin, SmartUpdateView):
        success_url = "@stories.story_list"
        title = _("Story Images")

        def get_form(self):
            form = super(StoryCRUDL.Images, self).get_form()
            form.fields.clear()

            idx = 1

            # add existing images
            for image in self.object.images.all().order_by("pk"):
                image_field_name = "image_%d" % idx
                image_field = forms.ImageField(
                    required=False,
                    initial=image.image,
                    label=_("Image %d") % idx,
                    help_text=_("Image to display on story page and in previews. (optional)"),
                    validators=[validate_image_file_extension],
                )

                self.form.fields[image_field_name] = image_field
                idx += 1

            while idx <= 3:
                self.form.fields["image_%d" % idx] = forms.ImageField(
                    required=False,
                    label=_("Image %d") % idx,
                    help_text=_("Image to display on story page and in previews (optional)"),
                    validators=[validate_image_file_extension],
                )
                idx += 1

            return form

        def post_save(self, obj):
            obj = super(StoryCRUDL.Images, self).post_save(obj)

            # remove our existing images
            self.object.images.all().delete()

            # overwrite our new ones
            # TODO: this could probably be done more elegantly
            for idx in range(1, 4):
                image = self.form.cleaned_data.get("image_%d" % idx, None)

                if image:
                    StoryImage.objects.create(
                        story=self.object, image=image, created_by=self.request.user, modified_by=self.request.user
                    )

            return obj

    class Create(OrgPermsMixin, SmartCreateView):
        form_class = StoryForm
        success_url = "id@stories.story_images"
        fields = (
            "title",
            "featured",
            "summary",
            "content",
            "attachment",
            "written_by",
            "audio_link",
            "video_id",
            "tags",
            "category",
        )

        def pre_save(self, obj):
            obj = super(StoryCRUDL.Create, self).pre_save(obj)

            obj.org = self.request.org
            obj.audio_link = Story.format_audio_link(obj.audio_link)
            obj.tags = Story.space_tags(obj.tags)
            return obj

        def get_form_kwargs(self):
            kwargs = super(StoryCRUDL.Create, self).get_form_kwargs()
            kwargs["org"] = self.request.org
            return kwargs



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL), ("orgs", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="DashBlock",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                (
                    "title",
                    models.CharField(
                        help_text="The title for this block of content, optional",
                        max_length=255,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "summary",
                    models.TextField(help_text="The summary for this item, should be short", null=True, blank=True),
                ),
                (
                    "content",
                    models.TextField(
                        help_text="The body of text for this content block, optional", null=True, blank=True
                    ),
                ),
                (
                    "image",
                    models.ImageField(
                        help_text="Any image that should be displayed with this content block, optional",
                        null=True,
                        upload_to="dashblocks",
                        blank=True,
                    ),
                ),
                (
                    "color",
                    models.CharField(
                        help_text="A background color to use for the image, in the format: #rrggbb",
                        max_length=16,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "link",
                    models.CharField(
                        help_text="Any link that should be associated with this content block, optional",
                        max_length=255,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "video_id",
                    models.CharField(
                        help_text="The id of the YouTube video that should be linked to this item",
                        max_length=255,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "tags",
                    models.CharField(
                        help_text="Any tags for this content block, separated by spaces, can be used to do more advanced filtering, optional",
                        max_length=255,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "priority",
                    models.IntegerField(
                        default=0, help_text="The priority for this block, higher priority blocks come first"
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "org",
                    models.ForeignKey(
                        help_text="The organization this content block belongs to",
                        on_delete=models.PROTECT,
                        to="orgs.Org",
                    ),
                ),
            ],
            options={"abstract": False},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="DashBlockImage",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                (
                    "image",
                    models.ImageField(height_field="height", width_field="width", upload_to="dashblock_images/"),
                ),
                ("caption", models.CharField(max_length=64)),
                ("priority", models.IntegerField(default=0, null=True, blank=True)),
                ("width", models.IntegerField()),
                ("height", models.IntegerField()),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("dashblock", models.ForeignKey(to="dashblocks.DashBlock", on_delete=models.PROTECT)),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"abstract": False},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="DashBlockType",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                (
                    "name",
                    models.CharField(
                        help_text="The human readable name for this content type", unique=True, max_length=75
                    ),
                ),
                (
                    "slug",
                    models.SlugField(
                        help_text="The slug to idenfity this content type, used with the template tags", unique=True
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        help_text="A description of where this content type is used on the site and how it will be dsiplayed",
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "has_title",
                    models.BooleanField(default=True, help_text="Whether this content should include a title"),
                ),
                (
                    "has_image",
                    models.BooleanField(default=True, help_text="Whether this content should include an image"),
                ),
                (
                    "has_rich_text",
                    models.BooleanField(default=True, help_text="Whether this content should use a rich HTML editor"),
                ),
                (
                    "has_summary",
                    models.BooleanField(default=True, help_text="Whether this content should include a summary field"),
                ),
                (
                    "has_link",
                    models.BooleanField(default=True, help_text="Whether this content should include a link"),
                ),
                (
                    "has_gallery",
                    models.BooleanField(
                        default=False,
                        help_text="Whether this content should allow upload of additional images, ie a gallery",
                    ),
                ),
                ("has_color", models.BooleanField(default=False, help_text="Whether this content has a color field")),
                (
                    "has_video",
                    models.BooleanField(
                        default=False, help_text="Whether this content should allow setting a YouTube id"
                    ),
                ),
                ("has_tags", models.BooleanField(default=False, help_text="Whether this content should allow tags")),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"abstract": False},
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name="dashblock",
            name="dashblock_type",
            field=models.ForeignKey(
                verbose_name="Content Type",
                on_delete=models.PROTECT,
                to="dashblocks.DashBlockType",
                help_text="The category, or type for this content block",
            ),
            preserve_default=True,
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-15 15:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("orgs", "0027_alter_org_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="config",
            field=models.JSONField(
                default=dict,
                help_text="JSON blob used to store configuration information associated with this organization",
            ),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("categories", "0002_auto_20140820_1415"), ("stories", "0004_auto_20140806_1540")]

    operations = [
        migrations.AddField(
            model_name="story",
            name="category",
            field=models.ForeignKey(
                blank=True,
                to="categories.Category",
                on_delete=models.PROTECT,
                help_text="The category for this story",
                null=True,
            ),
            preserve_default=True,
        )
    ]



import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("categories", "0006_auto_20141008_1955")]

    operations = [
        migrations.AlterField(
            model_name="category",
            name="created_on",
            field=models.DateTimeField(
                blank=True,
                default=django.utils.timezone.now,
                editable=False,
                help_text="When this item was originally created",
            ),
        ),
        migrations.AlterField(
            model_name="category",
            name="modified_on",
            field=models.DateTimeField(
                blank=True,
                default=django.utils.timezone.now,
                editable=False,
                help_text="When this item was last modified",
            ),
        ),
        migrations.AlterField(
            model_name="categoryimage",
            name="created_on",
            field=models.DateTimeField(
                blank=True,
                default=django.utils.timezone.now,
                editable=False,
                help_text="When this item was originally created",
            ),
        ),
        migrations.AlterField(
            model_name="categoryimage",
            name="modified_on",
            field=models.DateTimeField(
                blank=True,
                default=django.utils.timezone.now,
                editable=False,
                help_text="When this item was last modified",
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-10 14:50

import functools

from django.db import migrations, models

import dash.utils


class Migration(migrations.Migration):

    dependencies = [
        ("dashblocks", "0008_auto_20210902_1125"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dashblock",
            name="image",
            field=models.ImageField(
                blank=True,
                help_text="Any image that should be displayed with this content block, optional",
                null=True,
                upload_to=functools.partial(dash.utils.generate_file_path, *("dashblocks",), **{}),
            ),
        ),
        migrations.AlterField(
            model_name="dashblockimage",
            name="image",
            field=models.ImageField(
                height_field="height",
                upload_to=functools.partial(dash.utils.generate_file_path, *("dashblock_images/",), **{}),
                width_field="width",
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("categories", "0005_auto_20140922_1514")]

    operations = [
        migrations.AlterField(
            model_name="category",
            name="org",
            field=models.ForeignKey(
                related_name="categories",
                to="orgs.Org",
                on_delete=models.PROTECT,
                help_text="The organization this category applies to",
            ),
        )
    ]



import os
from functools import partial

from smartmin.models import SmartModel

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Prefetch, Q
from django.utils.translation import gettext_lazy as _

from dash.categories.models import Category
from dash.orgs.models import Org
from dash.utils import generate_file_path


def validate_file_extension(value):
    ext = os.path.splitext(value.name)[1]
    valid_extensions = [".pdf"]
    if ext not in valid_extensions:
        raise ValidationError(_("Only PDF files are supported."))


class Story(SmartModel):
    title = models.CharField(max_length=255, help_text=_("The title for this story"))

    featured = models.BooleanField(default=False, help_text=_("Whether this story is featured"))
    summary = models.TextField(null=True, blank=True, help_text=_("The summary for the story"))

    content = models.TextField(help_text=_("The body of text for the story"))

    written_by = models.CharField(max_length=255, help_text=_("The writer of the story"), null=True, blank=True)

    audio_link = models.URLField(
        max_length=255, blank=True, null=True, help_text=_("A link to an mp3 file to publish on this story")
    )

    attachment = models.FileField(
        upload_to=partial(generate_file_path, "story_attachments"),
        blank=True,
        null=True,
        validators=[validate_file_extension],
        help_text=_("The PDF report to attach"),
    )

    video_id = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        help_text=_(
            "The id of the YouTube video that should be linked to "
            "this story (this is the text that comes afer v= and "
            "before & in the YouTube URL)"
        ),
    )

    tags = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        help_text=_(
            "Any tags for this story, separated by spaces, can be " "used to do more advanced filtering, optional"
        ),
    )

    category = models.ForeignKey(
        Category, on_delete=models.PROTECT, null=True, blank=True, help_text=_("The category for this story")
    )

    org = models.ForeignKey(Org, on_delete=models.PROTECT, help_text=_("The organization this story belongs to"))

    @classmethod
    def format_audio_link(cls, link):
        formatted_link = link
        if not formatted_link:
            return formatted_link

        if formatted_link == "http://":
            return ""

        if not formatted_link.startswith("http://"):
            formatted_link = "http://" + formatted_link
        return formatted_link

    @classmethod
    def space_tags(cls, tags):
        """
        If we have tags set, then adds spaces before and after to allow for SQL
        querying for them.
        """
        if tags and tags.strip():
            return " " + tags.strip().lower() + " "

    def teaser(self, field, length):
        if not field:
            return ""
        words = field.split(" ")

        if len(words) < length:
            return field
        else:
            return " ".join(words[:length]) + " .."

    def long_teaser(self):
        if self.summary:
            return self.teaser(self.summary, 100)
        return self.teaser(self.content, 100)

    def short_teaser(self):
        if self.summary:
            return self.teaser(self.summary, 40)
        return self.teaser(self.content, 40)

    def get_written_by(self):
        if self.written_by:
            return self.written_by

        if self.created_by.first_name or self.created_by.last_name:
            full_name = self.created_by.first_name + " " + self.created_by.last_name
            return full_name.strip()

    def get_featured_images(self):
        return (
            self.prefetched_images
            if hasattr(self, "prefetched_images")
            else self.images.filter(is_active=True).exclude(image="")
        )

    def get_category_image(self):
        cat_image = None
        if self.category and self.category.is_active:
            cat_image = self.category.get_first_image()

        if not cat_image:
            if self.get_featured_images():
                cat_image = self.get_featured_images()[0].image

        return cat_image

    def get_image(self):
        cat_image = None
        if self.get_featured_images():
            cat_image = self.get_featured_images()[0].image

        if not cat_image:
            if self.category and self.category.is_active:
                cat_image = self.category.get_first_image()

        return cat_image

    @classmethod
    def get_main_stories(cls, org, limit=None):
        stories = (
            Story.objects.filter(org=org, featured=True, is_active=True)
            .filter(Q(attachment="") | Q(attachment=None))
            .prefetch_related(
                Prefetch(
                    "images",
                    queryset=StoryImage.objects.filter(is_active=True).exclude(image=""),
                    to_attr="prefetched_images",
                )
            )
            .order_by("-created_on")
        )

        return stories[:limit] if limit else stories

    class Meta:
        verbose_name_plural = _("Stories")


class StoryImage(SmartModel):
    name = models.CharField(max_length=64, help_text=_("The name to describe this image"))

    story = models.ForeignKey(
        Story, on_delete=models.PROTECT, related_name="images", help_text=_("The story to associate to")
    )

    image = models.ImageField(upload_to=partial(generate_file_path, "stories"), help_text=_("The image file to use"))



# Generated by Django 4.0.1 on 2022-01-12 15:46

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("dashblocks", "0009_auto_20210910_1450"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dashblock",
            name="created_by",
            field=models.ForeignKey(
                help_text="The user which originally created this item",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="%(app_label)s_%(class)s_creations",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="dashblock",
            name="modified_by",
            field=models.ForeignKey(
                help_text="The user which last modified this item",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="%(app_label)s_%(class)s_modifications",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="dashblockimage",
            name="created_by",
            field=models.ForeignKey(
                help_text="The user which originally created this item",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="%(app_label)s_%(class)s_creations",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="dashblockimage",
            name="modified_by",
            field=models.ForeignKey(
                help_text="The user which last modified this item",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="%(app_label)s_%(class)s_modifications",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="dashblocktype",
            name="created_by",
            field=models.ForeignKey(
                help_text="The user which originally created this item",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="%(app_label)s_%(class)s_creations",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="dashblocktype",
            name="modified_by",
            field=models.ForeignKey(
                help_text="The user which last modified this item",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="%(app_label)s_%(class)s_modifications",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0002_auto_20140802_2104")]

    operations = [
        migrations.AddField(
            model_name="org",
            name="logo",
            field=models.ImageField(
                help_text="The logo that should be used for this organization",
                null=True,
                upload_to="logos",
                blank=True,
            ),
            preserve_default=True,
        )
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0015_auto_20160209_0926")]

    operations = [
        migrations.AddField(model_name="taskstate", name="is_disabled", field=models.BooleanField(default=False))
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-27 12:04


from django.db import migrations


class Migration(migrations.Migration):
    def fix_org_config_rapidpro(apps, schema_editor):
        Org = apps.get_model("orgs", "Org")
        orgs = Org.objects.all()
        for org in orgs:
            if not org.config:
                continue

            config = org.config
            rapidpro_config = config.get("rapidpro", dict())
            if "api_token" in rapidpro_config:
                del rapidpro_config["api_token"]

            if "rapipro" in config:
                del config["rapipro"]  # remove the mistakenly added key by typo in 0024_populate_org_backend

            config["rapidpro"] = rapidpro_config
            org.config = config
            org.save()

    def noop(apps, schema_editor):
        pass

    dependencies = [("orgs", "0025_auto_20180322_1415")]

    operations = [migrations.RunPython(fix_org_config_rapidpro, noop)]



import json
import zoneinfo
from datetime import datetime
from itertools import chain

from django.core.cache import cache
from django.utils import timezone

from dash.test import DashTest

from . import (
    chunks,
    datetime_to_ms,
    filter_dict,
    get_cacheable,
    get_month_range,
    get_obj_cacheable,
    intersection,
    is_dict_equal,
    ms_to_datetime,
    random_string,
    union,
)


class InitTest(DashTest):
    def test_intersection(self):
        self.assertEqual(intersection(), [])
        self.assertEqual(intersection([1]), [1])
        self.assertEqual(intersection([2, 1, 1]), [2, 1])
        self.assertEqual(intersection([3, 2, 1], [2, 3, 4]), [3, 2])  # order from first list
        self.assertEqual(intersection([4, 3, 2, 1], [3, 2, 4], [1, 2, 3]), [3, 2])

    def test_union(self):
        self.assertEqual(union(), [])
        self.assertEqual(union([1]), [1])
        self.assertEqual(union([2, 1, 1], [1, 2, 3]), [2, 1, 3])  # order is first seen
        self.assertEqual(union([2, 1], [2, 3, 3], [4, 5]), [2, 1, 3, 4, 5])

    def test_random_string(self):
        rs = random_string(1000)
        self.assertEqual(1000, len(rs))
        self.assertFalse("1" in rs or "I" in rs or "0" in rs or "O" in rs)

    def test_filter_dict(self):
        d = {"a": 123, "b": "xyz", "c": 456}
        self.assertEqual(filter_dict(d, ()), {})
        self.assertEqual(filter_dict(d, ("a", "c")), {"a": 123, "c": 456})

    def test_get_cacheable(self):
        def calculate1():
            return "CALCULATED"

        self.assertEqual(get_cacheable("test_key:1", 60, calculate1), "CALCULATED")
        cache.set("test_key:1", json.dumps("CACHED"), 60)
        self.assertEqual(get_cacheable("test_key:1", 60, calculate1), "CACHED")
        self.assertEqual(get_cacheable("test_key:1", 60, calculate1, recalculate=True), "CALCULATED")

        # falsey values shouldn't trigger re-calculation
        cache.set("test_key:1", json.dumps(0), 60)
        self.assertEqual(get_cacheable("test_key:1", 60, calculate1), 0)

        def calculate2():
            return dict(a=123, b="abc")

        self.assertEqual(get_cacheable("test_key:2", 60, calculate2), dict(a=123, b="abc"))
        cache.set("test_key:2", '{"a":234,"b":"xyz"}', 60)
        self.assertEqual(get_cacheable("test_key:2", 60, calculate2), dict(a=234, b="xyz"))

    def test_get_obj_cacheable(self):
        def calculate():
            return "CALCULATED"

        self.assertEqual(get_obj_cacheable(self, "_test_value", calculate), "CALCULATED")
        self._test_value = "CACHED"
        self.assertEqual(get_obj_cacheable(self, "_test_value", calculate), "CACHED")
        self.assertEqual(get_obj_cacheable(self, "_test_value", calculate, recalculate=True), "CALCULATED")

    def test_datetime_to_ms(self):
        d1 = datetime(2014, 1, 2, 3, 4, 5, 678900, tzinfo=timezone.utc)
        self.assertEqual(datetime_to_ms(d1), 1388631845678)  # from http://unixtimestamp.50x.eu

        # conversion to millis loses some accuracy
        self.assertEqual(ms_to_datetime(1388631845678), datetime(2014, 1, 2, 3, 4, 5, 678000, tzinfo=timezone.utc))

        tz = zoneinfo.ZoneInfo("Africa/Kigali")
        d2 = datetime(2014, 1, 2, 3, 4, 5, 600000, tz)
        self.assertEqual(datetime_to_ms(d2), 1388624645600)
        self.assertEqual(ms_to_datetime(1388624645600), d2.astimezone(timezone.utc))

    def test_get_month_range(self):
        self.assertEqual(
            get_month_range(datetime(2014, 2, 10, 12, 30, 0, 0, zoneinfo.ZoneInfo("Africa/Kigali"))),
            (
                datetime(2014, 2, 1, 0, 0, 0, 0, zoneinfo.ZoneInfo("Africa/Kigali")),
                datetime(2014, 3, 1, 0, 0, 0, 0, zoneinfo.ZoneInfo("Africa/Kigali")),
            ),
        )

    def test_chunks(self):
        self.assertEqual(list(chunks([], 2)), [])
        self.assertEqual(list(chunks([1, 2, 3, 4, 5], 2)), [[1, 2], [3, 4], [5]])

        # if data is a set, chunking still works but with non-deterministic ordering
        batches = list(chunks({1, 2, 3, 4, 5}, 2))
        self.assertEqual(len(batches), 3)
        self.assertEqual(set(chain(*batches)), {1, 2, 3, 4, 5})

    def test_is_dict_equal(self):
        self.assertTrue(is_dict_equal({"a": 1, "b": 2}, {"b": 2, "a": 1}))
        self.assertFalse(is_dict_equal({"a": 1, "b": 2}, {"a": 1, "b": 3}))
        self.assertFalse(is_dict_equal({"a": 1, "b": 2}, {"a": 1, "c": 2}))
        self.assertFalse(is_dict_equal({"a": 1, "b": 2}, {"a": 1, "b": 2, "c": 3}))

        self.assertTrue(is_dict_equal({"a": 1, "b": 2, "c": 3}, {"a": 1, "b": 2, "c": 4}, keys=("a", "b")))

        self.assertTrue(is_dict_equal({"a": 1, "b": 2}, {"a": 1, "b": 2, "c": None}, ignore_none_values=True))
        self.assertFalse(is_dict_equal({"a": 1, "b": 2}, {"a": 1, "b": 2, "c": None}, ignore_none_values=False))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-15 12:24


from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("orgs", "0021_auto_20180315_0823")]

    def populate_rapidpro_config(apps, schema_editor):
        Org = apps.get_model("orgs", "Org")
        orgs = Org.objects.all()

        for org in orgs:
            if not org.config:
                continue

            backends_config_dict = getattr(settings, "DATA_API_BACKENDS_CONFIG", {})
            for backend_slug in backends_config_dict:
                config_fields = getattr(settings, "BACKENDS_ORG_CONFIG_FIELDS", [])
                for config_field in config_fields:
                    name = config_field["name"]
                    existing_value = org.config["common"].get(name, None)
                    if backend_slug not in org.config:
                        org.config[backend_slug] = dict()
                    org.config[backend_slug][name] = existing_value

            org.save()

    def noop(apps, schema_editor):
        pass

    operations = [migrations.RunPython(populate_rapidpro_config, noop)]



# Generated by Django 3.2.6 on 2021-09-10 14:50

import functools

from django.db import migrations, models

import dash.utils


class Migration(migrations.Migration):

    dependencies = [
        ("categories", "0008_alter_category_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="category",
            name="image",
            field=models.ImageField(
                blank=True,
                help_text="An optional image that can describe this category",
                null=True,
                upload_to=functools.partial(dash.utils.generate_file_path, *("categories",), **{}),
            ),
        ),
        migrations.AlterField(
            model_name="categoryimage",
            name="image",
            field=models.ImageField(
                help_text="The image file to use",
                upload_to=functools.partial(dash.utils.generate_file_path, *("categories",), **{}),
            ),
        ),
    ]



import json
import random
from functools import partial
from pydoc import locate

from smartmin.models import SmartModel
from temba_client.v2 import TembaClient
from timezone_field import TimeZoneField

from django.conf import settings
from django.contrib.auth.models import Group, User
from django.db import models
from django.utils import timezone
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from dash.utils import generate_file_path
from dash.utils.email import send_dash_email

STATE = 1
DISTRICT = 2

# we cache boundary data for a month at a time
BOUNDARY_CACHE_TIME = getattr(settings, "API_BOUNDARY_CACHE_TIME", 60 * 60 * 24 * 30)

BOUNDARY_CACHE_KEY = "org:%d:boundaries"
BOUNDARY_LEVEL_1_KEY = "geojson:%d"
BOUNDARY_LEVEL_2_KEY = "geojson:%d:%s"


class Org(SmartModel):
    name = models.CharField(verbose_name=_("Name"), max_length=128, help_text=_("The name of this organization"))

    logo = models.ImageField(
        upload_to=partial(generate_file_path, "logos"),
        null=True,
        blank=True,
        help_text=_("The logo that should be used for this organization"),
    )

    administrators = models.ManyToManyField(
        User,
        verbose_name=_("Administrators"),
        related_name="org_admins",
        help_text=_("The administrators in your organization"),
    )

    viewers = models.ManyToManyField(
        User, verbose_name=_("Viewers"), related_name="org_viewers", help_text=_("The viewers in your organization")
    )

    editors = models.ManyToManyField(
        User, verbose_name=_("Editors"), related_name="org_editors", help_text=_("The editors in your organization")
    )

    language = models.CharField(
        verbose_name=_("Language"),
        max_length=64,
        null=True,
        blank=True,
        help_text=_("The main language used by this organization"),
    )

    subdomain = models.SlugField(
        verbose_name=_("Subdomain"),
        null=True,
        blank=True,
        max_length=255,
        unique=True,
        error_messages=dict(unique=_("This subdomain is not available")),
        help_text=_("The subdomain for this organization"),
    )

    domain = models.CharField(
        verbose_name=_("Domain"),
        null=True,
        blank=True,
        max_length=255,
        unique=True,
        error_messages=dict(unique=_("This domain is not available")),
        help_text=_("The custom domain for this organization"),
    )

    timezone = TimeZoneField(
        verbose_name=_("Timezone"), default="UTC", help_text=_("The timezone your organization is in.")
    )

    config = models.JSONField(
        default=dict,
        help_text=_("JSON blob used to store configuration information " "associated with this organization"),
    )

    def get_backend(self, backend_slug="rapidpro"):
        backend = self.backends.filter(is_active=True, slug=backend_slug).first()
        return locate(backend.backend_type)(backend=backend)

    def get_config(self, name, default=None):
        config = getattr(self, "_config", None)

        if config is None:
            if not self.config:
                return default

            config = self.config
            self._config = config

        if name.find(".") == -1:
            name = "common.%s" % name

        key1, key2 = name.split(".", 1)
        return config.get(key1, dict()).get(key2, default)

    def set_config(self, name, value, commit=True):
        if not self.config:
            config = dict()
        else:
            config = self.config

        if name.find(".") == -1:
            name = "common.%s" % name
        key1, key2 = name.split(".", 1)

        if key1 not in config:
            config[key1] = dict()

        config[key1][key2] = value
        self.config = config
        self._config = config

        if commit:
            self.save()

    def get_org_admins(self):
        return self.administrators.all()

    def get_org_editors(self):
        return self.editors.all()

    def get_org_viewers(self):
        return self.viewers.all()

    def get_org_users(self):
        org_users = self.get_org_admins() | self.get_org_editors() | self.get_org_viewers()
        return org_users.distinct()

    def get_user_org_group(self, user):
        if user in self.get_org_admins():
            user._org_group = Group.objects.get(name="Administrators")
        elif user in self.get_org_editors():
            user._org_group = Group.objects.get(name="Editors")
        elif user in self.get_org_viewers():
            user._org_group = Group.objects.get(name="Viewers")
        else:
            user._org_group = None

        return getattr(user, "_org_group", None)

    def get_user(self):
        user = self.administrators.filter(is_active=True).first()
        if user:
            org_user = user
            org_user.set_org(self)
            return org_user

    def get_temba_client(self, api_version=2):
        if api_version not in (2,):
            raise ValueError("Unsupported RapidPro API version: %d" % api_version)

        host = getattr(settings, "SITE_API_HOST", None)
        agent = getattr(settings, "SITE_API_USER_AGENT", None)

        if host.endswith("api/v2") or host.endswith("api/v2/"):
            raise ValueError(
                "API host should not include API version, "
                "e.g. http://example.com instead of http://example.com/api/v2"
            )

        api_token = ""
        backend = self.backends.filter(is_active=True, slug="rapidpro").first()
        if backend:
            api_token = backend.api_token
            if backend.host:
                host = backend.host

        return TembaClient(host, api_token, user_agent=agent)

    def build_host_link(self, user_authenticated=False):
        host_tld = getattr(settings, "HOSTNAME", "locahost")
        is_secure = getattr(settings, "SESSION_COOKIE_SECURE", False)

        prefix = "http://"

        if self.domain and is_secure and not user_authenticated:
            return prefix + str(self.domain)

        if is_secure:
            prefix = "https://"

        if self.subdomain == "":
            return prefix + host_tld
        return prefix + force_str(self.subdomain) + "." + host_tld

    def get_task_state(self, task_key):
        return TaskState.get_or_create(self, task_key)

    @classmethod
    def create_user(cls, email, password):
        user = User.objects.create_user(username=email, email=email, password=password)
        return user

    @classmethod
    def get_org(cls, user):
        if not user:
            return None

        if not hasattr(user, "_org"):
            org = Org.objects.filter(administrators=user, is_active=True).first()
            if org:
                user._org = org

        return getattr(user, "_org", None)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


def get_org(obj):
    return getattr(obj, "_org", None)


def set_org(obj, org):
    obj._org = org


def get_user_orgs(user):
    if user.is_superuser:
        return Org.objects.all()
    user_orgs = user.org_admins.all() | user.org_editors.all() | user.org_viewers.all()
    return user_orgs.distinct()


def get_org_group(obj):
    org_group = None
    org = obj.get_org()
    if org:
        org_group = org.get_user_org_group(obj)
    return org_group


User.get_org = get_org
User.set_org = set_org
User.get_user_orgs = get_user_orgs
User.get_org_group = get_org_group


USER_GROUPS = (("A", _("Administrator")), ("E", _("Editor")), ("V", _("Viewer")))


class Invitation(SmartModel):
    org = models.ForeignKey(
        Org,
        on_delete=models.PROTECT,
        verbose_name=_("Org"),
        related_name="invitations",
        help_text=_("The organization to which the account is invited to view"),
    )

    email = models.EmailField(
        verbose_name=_("Email"), help_text=_("The email to which we send the invitation of the viewer")
    )

    secret = models.CharField(
        verbose_name=_("Secret"),
        max_length=64,
        unique=True,
        help_text=_("a unique code associated with this invitation"),
    )

    user_group = models.CharField(max_length=1, choices=USER_GROUPS, default="V", verbose_name=_("User Role"))

    def save(self, *args, **kwargs):
        if not self.secret:
            secret = Invitation.generate_random_string(64)

            while Invitation.objects.filter(secret=secret):
                secret = Invitation.generate_random_string(64)

            self.secret = secret

        return super(Invitation, self).save(*args, **kwargs)

    @classmethod
    def generate_random_string(cls, length):
        """
        Generatesa a [length] characters alpha numeric secret
        """
        # avoid things that could be mistaken ex: 'I' and '1'
        letters = "23456789ABCDEFGHJKLMNPQRSTUVWXYZ"
        return "".join([random.choice(letters) for _ in range(length)])

    def send_invitation(self):
        from .tasks import send_invitation_email_task

        send_invitation_email_task(self.id)

    def send_email(self):
        # no=op if we do not know the email
        if not self.email:
            return

        subject = _("%s Invitation") % self.org.name
        template = "orgs/email/invitation_email"
        to_email = self.email

        context = dict(org=self.org, now=timezone.now(), invitation=self)
        context["subject"] = subject
        context["host"] = self.org.build_host_link()

        send_dash_email(to_email, subject, template, context)


class OrgBackground(SmartModel):
    BACKGROUND_TYPES = (("B", _("Banner")), ("P", _("Pattern")))

    org = models.ForeignKey(
        Org,
        on_delete=models.PROTECT,
        verbose_name=_("Org"),
        related_name="backgrounds",
        help_text=_("The organization in which the image will be used"),
    )

    name = models.CharField(
        verbose_name=_("Name"), max_length=128, help_text=_("The name to describe this background")
    )

    background_type = models.CharField(
        max_length=1, choices=BACKGROUND_TYPES, default="P", verbose_name=_("Background type")
    )

    image = models.ImageField(upload_to=partial(generate_file_path, "org_bgs"), help_text=_("The image file"))


class TaskState(models.Model):
    """
    Holds org specific state for a scheduled task
    """

    LOCK_KEY = "org-task-lock:%s:%s"

    org = models.ForeignKey(Org, on_delete=models.PROTECT, related_name="task_states")

    task_key = models.CharField(max_length=32)

    started_on = models.DateTimeField(null=True)

    ended_on = models.DateTimeField(null=True)

    last_successfully_started_on = models.DateTimeField(null=True)

    last_results = models.TextField(null=True)

    is_failing = models.BooleanField(default=False)

    is_disabled = models.BooleanField(default=False)

    @classmethod
    def get_or_create(cls, org, task_key):
        existing = cls.objects.filter(org=org, task_key=task_key).first()
        if existing:
            return existing

        return cls.objects.create(org=org, task_key=task_key)

    @classmethod
    def get_lock_key(cls, org, task_key):
        return cls.LOCK_KEY % (org.id, task_key)

    @classmethod
    def get_failing(cls):
        return cls.objects.filter(org__is_active=True, is_failing=True)

    def is_running(self):
        return self.started_on and not self.ended_on

    def has_ever_run(self):
        return self.started_on is not None

    def get_last_results(self):
        return json.loads(self.last_results) if self.last_results else None

    def get_time_taken(self):
        until = self.ended_on if self.ended_on else timezone.now()
        return (until - self.started_on).total_seconds()

    class Meta:
        unique_together = ("org", "task_key")


class OrgBackend(SmartModel):
    org = models.ForeignKey(Org, on_delete=models.PROTECT, related_name="backends")

    slug = models.CharField(max_length=16)

    backend_type = models.CharField(max_length=256)

    host = models.CharField(max_length=128)

    api_token = models.CharField(max_length=128, help_text=_("The API token for this backend"))

    def __str__(self):
        return self.slug

    class Meta:
        unique_together = ("org", "slug")



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("stories", "0002_auto_20140805_1158")]

    operations = [
        migrations.AddField(
            model_name="story",
            name="summary",
            field=models.TextField(help_text="The summary for the story", null=True, blank=True),
            preserve_default=True,
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-21 09:57


from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("orgs", "0023_orgbackend")]

    def populate_org_backend(apps, schema_editor):
        Org = apps.get_model("orgs", "Org")
        OrgBackend = apps.get_model("orgs", "OrgBackend")
        orgs = Org.objects.all()
        User = apps.get_model("auth", "User")
        root = User.objects.filter(username="root").first()

        if not root:
            root = User.objects.filter(username="root2").first()

        if not root:
            root = User.objects.create(username="root2")

        default_backend = getattr(settings, "SITE_BACKEND", None)
        host = getattr(settings, "SITE_API_HOST", None)

        for org in orgs:
            if not org.config:
                continue

            config = org.config
            rapidpro_config = config.get("rapidpro", dict())
            api_token = rapidpro_config.get("api_token", "")
            OrgBackend.objects.create(
                org=org,
                slug="rapidpro",
                api_token=api_token,
                host=host,
                backend_type=default_backend,
                created_by=root,
                modified_by=root,
            )

            del rapidpro_config["api_token"]
            config["rapipro"] = rapidpro_config
            org.config = config
            org.save()

    def noop(apps, schema_editor):
        pass

    operations = [migrations.RunPython(populate_org_backend, noop)]



from functools import partial

from smartmin.models import SmartModel

from django.db import models
from django.utils.translation import gettext_lazy as _

from dash.orgs.models import Org
from dash.utils import generate_file_path


class Category(SmartModel):
    """
    Every organization can choose to categorize their polls or stories
    according to their needs.
    """

    name = models.CharField(max_length=64, help_text=_("The name of this category"))

    image = models.ImageField(
        upload_to=partial(generate_file_path, "categories"),
        null=True,
        blank=True,
        help_text=_("An optional image that can describe this category"),
    )

    org = models.ForeignKey(
        Org,
        on_delete=models.PROTECT,
        related_name="categories",
        help_text=_("The organization this category applies to"),
    )

    def get_first_image(self):
        cat_images = self.images.filter(is_active=True).exclude(image="")
        if cat_images and cat_images.first().image:
            return cat_images.first().image

    def get_label_from_instance(self):
        label = str(self)
        if isinstance(label, bytes):
            label = label.decode("utf-8")

        if not self.is_active:
            label = "%s %s" % (label, "(Inactive)")
        return label

    def __str__(self):
        return "%s - %s" % (self.org, self.name)

    class Meta:
        ordering = ["name"]
        unique_together = ("name", "org")
        verbose_name_plural = _("Categories")


class CategoryImage(SmartModel):
    name = models.CharField(max_length=64, help_text=_("The name to describe this image"))

    category = models.ForeignKey(
        Category, on_delete=models.PROTECT, related_name="images", help_text=_("The category this image represents")
    )

    image = models.ImageField(
        upload_to=partial(generate_file_path, "categories"), help_text=_("The image file to use")
    )

    def __str__(self):
        return "%s - %s" % (self.category.name, self.name)



# Generated by Django 3.2.6 on 2021-10-25 15:04

import functools

from django.db import migrations, models

import dash.utils


class Migration(migrations.Migration):

    dependencies = [
        ("orgs", "0028_alter_org_config"),
    ]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="logo",
            field=models.ImageField(
                blank=True,
                help_text="The logo that should be used for this organization",
                null=True,
                upload_to=functools.partial(dash.utils.generate_file_path, *("logos",), **{}),
            ),
        ),
        migrations.AlterField(
            model_name="orgbackground",
            name="image",
            field=models.ImageField(
                help_text="The image file",
                upload_to=functools.partial(dash.utils.generate_file_path, *("org_bgs",), **{}),
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-02 11:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("categories", "0007_auto_20170301_0914"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="category",
            options={"ordering": ["name"], "verbose_name_plural": "Categories"},
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dash',
 'dash.categories',
 'dash.categories.migrations',
 'dash.dashblocks',
 'dash.dashblocks.migrations',
 'dash.dashblocks.templatetags',
 'dash.orgs',
 'dash.orgs.management',
 'dash.orgs.management.commands',
 'dash.orgs.migrations',
 'dash.orgs.templatetags',
 'dash.stories',
 'dash.stories.migrations',
 'dash.tags',
 'dash.tags.migrations',
 'dash.users',
 'dash.utils',
 'dash.utils.management',
 'dash.utils.management.commands',
 'dash.utils.templatetags']

package_data = \
{'': ['*'],
 'dash.categories': ['templates/categories/*'],
 'dash.dashblocks': ['static/css/*',
                     'static/css/font/*',
                     'static/js/*',
                     'templates/dashblocks/*'],
 'dash.orgs': ['templates/*', 'templates/orgs/*', 'templates/orgs/email/*'],
 'dash.stories': ['templates/stories/*']}

install_requires = \
['Django>=4.0,<5.0',
 'Pillow>=9.0.0,<10.0.0',
 'celery>=5.1,<6.0',
 'django-compressor>=3.1,<4.0',
 'django-hamlpy>=1.4.3,<2.0.0',
 'django-redis>=4.12.1,<5.0.0',
 'django-timezone-field>=4.1.1,<5.0.0',
 'isort[pyproject]>=5.10.1,<6.0.0',
 'phonenumbers>=8.12.16,<9.0.0',
 'psycopg2-binary<2.9.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'rapidpro-python>=2.9.0,<3.0.0',
 'smartmin>=4.2.1,<5.0.0',
 'sorl-thumbnail>=12.7.0,<13.0.0']

setup_kwargs = {
    'name': 'rapidpro-dash',
    'version': '1.10.1',
    'description': 'Support library for RapidPro dashboards',
    'long_description': 'None',
    'author': 'Nyaruka Ltd',
    'author_email': 'code@nyaruka.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0014_auto_20150722_1419")]

    operations = [
        migrations.CreateModel(
            name="TaskState",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                ("task_key", models.CharField(max_length=32)),
                ("started_on", models.DateTimeField(null=True)),
                ("ended_on", models.DateTimeField(null=True)),
                ("last_successfully_started_on", models.DateTimeField(null=True)),
                ("last_results", models.TextField(null=True)),
                ("is_failing", models.BooleanField(default=False)),
                ("org", models.ForeignKey(related_name="task_states", on_delete=models.PROTECT, to="orgs.Org")),
            ],
        ),
        migrations.AlterUniqueTogether(name="taskstate", unique_together=set([("org", "task_key")])),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0007_auto_20140922_1514")]

    operations = [
        migrations.AddField(
            model_name="org",
            name="timezone",
            field=models.CharField(default="UTC", max_length=64, verbose_name="Timezone"),
            preserve_default=True,
        )
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0013_auto_20150715_1831")]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="domain",
            field=models.CharField(
                null=True,
                error_messages={"unique": "This domain is not available"},
                max_length=255,
                blank=True,
                help_text="The custom domain for this organization",
                unique=True,
                verbose_name="Domain",
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="subdomain",
            field=models.SlugField(
                null=True,
                error_messages={"unique": "This subdomain is not available"},
                max_length=255,
                blank=True,
                help_text="The subdomain for this organization",
                unique=True,
                verbose_name="Subdomain",
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="timezone",
            field=models.CharField(
                default="UTC",
                help_text="The timezone your organization is in.",
                max_length=64,
                verbose_name="Timezone",
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-10-26 18:01

import functools

from django.db import migrations, models

import dash.stories.models
import dash.utils


class Migration(migrations.Migration):

    dependencies = [
        ("stories", "0014_alter_storyimage_image"),
    ]

    operations = [
        migrations.AddField(
            model_name="story",
            name="attachment",
            field=models.FileField(
                blank=True,
                help_text="The PDF report to attach",
                null=True,
                upload_to=functools.partial(dash.utils.generate_file_path, *("story_attachments",), **{}),
                validators=[dash.stories.models.validate_file_extension],
            ),
        ),
    ]



from django.db import migrations


def pollcategory_to_category(apps, schema_editor):
    # Category = apps.get_model("categories", "Category")
    User = apps.get_model("auth", "User")
    # Org = apps.get_model("orgs", "Org")
    # from ureport.polls.models import PollCategory

    root = User.objects.filter(username="root").first()

    if not root:
        root = User.objects.filter(username="root2").first()

    if not root:
        root = User.objects.create(username="root2")
    #
    # for poll_cat in PollCategory.objects.all():
    #     org = Org.objects.get(pk=poll_cat.org.pk)
    #
    #     Category.objects.create(name=poll_cat.name,
    #                             org=org,
    #                             is_active=poll_cat.is_active,
    #                             created_by=root,
    #                             modified_by=root)


class Migration(migrations.Migration):

    dependencies = [("categories", "0001_initial")]

    operations = [migrations.RunPython(pollcategory_to_category)]
